<?php
namespace Teknisa\MYM\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Method;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\ParameterBag;
use Teknisa\Libs\Util\Utilities;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class Mym00106ParametrizacaoQrCodeController  extends \Teknisa\GER\Controllers\TeknisaCrud {
	
	protected $connection;
    protected $dataSourceName;
	protected $fileServerFilesUrl;
	
	public function __construct($crudService, ParameterBag $parameterBag, $connection, $fileServerFilesUrl) {
		parent::__construct($crudService, $parameterBag);
	    $this->connection = $connection;
		$this->fileServerFilesUrl = $fileServerFilesUrl;
	}
	
	private function existsParams($NRORG) {
		$query = "SELECT NRAPPCONFIG "
				."FROM APPCONFIG "
				."WHERE NRORG = :P_NRORG AND IDAPLICATIVO = 'MYMENU' ";
		$queryParams = [
			"P_NRORG" => $NRORG
		];
		
		$statement = $this->connection->prepare($query);
		$statement->execute($queryParams);
		$resultSet = $statement->fetchAll();
		
		return ($resultSet)? true : false;
	}
	
	private function getNextId($primaryKey, $table) {
		$nextId = null;
		
		$query = "SELECT MAX(".$primaryKey.") AS ".$primaryKey." "
 	       		."FROM ".$table;
    	    
	    $statement = $this->connection->prepare($query);
	    $statement->execute([]);
	    $resultSet = $statement->fetchAll();
	    
	    
	    if ($resultSet)
	    	$nextId = intval($resultSet[0][$primaryKey]) + 1;
	    	
	    return $nextId;
	}
	
	private function getDNSInfo($NRORG) {
	    $query = "SELECT NRAPPCONFIG, "
	    		."		 DSDNS, "
	    		."		 DSPORTA "
	            ."FROM APPCONFIG "
	            ."WHERE NRORG = :P_NRORG AND IDAPLICATIVO = 'MYMENU'";
        $queryParams = [
            "P_NRORG" => $NRORG
        ];
        
        $statement = $this->connection->prepare($query);
        $statement->execute($queryParams);
		$resultSet = $statement->fetchAll();
        if(!empty($resultSet)){
			return $resultSet[0];
		} else {
			return [];
		}
	}
	
	private function getQrCodeDescription($NRQRCODELOCALPESQ, $NRORG) {
		$query = "SELECT DSQRCODEAPP "
				."FROM QRCODELOCALPESQ "
				."WHERE NRQRCODELOCALPESQ = :P_NRQRCODELOCALPESQ "
				."	AND NRORG = :P_NRORG";
		$queryParams = [
			"P_NRQRCODELOCALPESQ"	=> $NRQRCODELOCALPESQ,
			"P_NRORG"				=> $NRORG
		];
		
		$statement = $this->connection->prepare($query);
		$statement->execute($queryParams);
		$resultSet = $statement->fetchAll();
		
		return ( isset($resultSet[0]["DSQRCODEAPP"]) )? $resultSet[0]["DSQRCODEAPP"] : NULL;
	}
	
	private function setQrCodeDescription($NRQRCODELOCALPESQ, $DSQRCODEAPP, $NRORG) {
		$query = "UPDATE QRCODELOCALPESQ "
				."SET DSQRCODEAPP = :P_DSQRCODEAPP "
				."WHERE NRQRCODELOCALPESQ = :P_NRQRCODELOCALPESQ "
				."	AND NRORG = :P_NRORG";
		$queryParams = [
			"P_NRQRCODELOCALPESQ"	=> $NRQRCODELOCALPESQ,
			"P_DSQRCODEAPP"			=> $DSQRCODEAPP,
			"P_NRORG"				=> $NRORG
		];
		
		$statement = $this->connection->prepare($query);
		$statement->execute($queryParams);
	}
	
	public function regenerateQrCodes(Request $request, Response $response) {
		
		try {
			$row = $request->getRow();
			
			foreach ($row['NRQRCODELOCALPESQ'] as $rowQrCodes) {
				$rowUpdate = array(
					"P_NRQRCODELOCALPESQ" => $rowQrCodes,
					"P_NRORG" => $row['NRORG']
				);
				$query = "UPDATE QRCODELOCALPESQ "
						."SET DSQRCODEAPP = (SELECT DSDNS ||';'||'80'||';'||:P_NRQRCODELOCALPESQ||';'||:P_NRORG FROM APPCONFIG WHERE IDAPLICATIVO = 'MYMENU' AND NRORG = :P_NRORG) "
						."WHERE NRQRCODELOCALPESQ = :P_NRQRCODELOCALPESQ AND NRORG = :P_NRORG";
				
				$statement = $this->connection->prepare($query);
				$statement->execute($rowUpdate);
			}
			
			
			$response->addDataSet(new DataSet("data",["QrCodes regerados com sucesso"]));
		} catch(\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
		
	}
	
	public function generateQrCode(Request $request, Response $response) {
		
		
		try {
		$row = $request->getRow();	
		if ($row["regenerate"]){
			$qrCodeDescription = null;
			$qrCodeDescriptionEncrypted = null;
		} else {
				$qrCodeDescription = $this->getQrCodeDescription($row["NRQRCODELOCALPESQ"], $row["NRORG"]);
				$qrCodeDescriptionEncrypted = $qrCodeDescription;
		}
	
		if (!$qrCodeDescription) {
			$appconfig = $this->getDNSInfo($row["NRORG"]);
			$intDsporta = intval($appconfig["DSPORTA"]);
			if(!empty($appconfig)){
				$qrCodeDescription = $appconfig["DSDNS"].";".$appconfig["DSPORTA"].";".$row["NRQRCODELOCALPESQ"].";".$row["NRORG"];
				
				$params = (object) array(
					"DSPORTA" => intval($appconfig["DSPORTA"]),
					"NRQRCODELOCALPESQ" => intval($row["NRQRCODELOCALPESQ"]),
					"NRORG" => intval($row["NRORG"])
				);
				
				$JSONParams = json_encode($params);
				//$qrCodeDescriptionEncrypted = $appconfig["DSDNS"]."/mobile/#/product#login?params=".Utilities::zeedhiEncrypt($JSONParams);
				$relativePathQrCode = InstanceManager::getInstance()->getParameter('relativePathQrCode');
				$qrCodeDescriptionEncrypted = $appconfig["DSDNS"].$relativePathQrCode."/#/product#login?params=".Utilities::zeedhiEncrypt($JSONParams);
			} else {
				$qrCodeDescription = "";
				$qrCodeDescriptionEncrypted = "";
			}
			
			$this->setQrCodeDescription($row["NRQRCODELOCALPESQ"], $qrCodeDescription, $row["NRORG"]);
		}

		$response->addDataSet(new DataSet("data",[$qrCodeDescription, $qrCodeDescriptionEncrypted]));
		} catch(\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function getAppConfig(Request $request, Response $response) {
		$row = $request->getRow();
	    
	    $resultSet = $this->getDNSInfo($row["NRORG"]);
        $response->addDataSet(new DataSet("data", $resultSet));
	}
	
	public function saveParams(Request $request, Response $response ) {
	    
	    try {
    	    $row = $request->getRow();
			
    	    if ($this->existsParams($row["NRORG"])) {
    	        $query = "UPDATE APPCONFIG "
    	        		."SET DSDNS = :P_DSDNS, "
    	        		."	  DSPORTA = :P_DSPORTA, "
    	        		."	  DTULTATU = :P_DTULTATU "
    	        		."WHERE NRORG = :P_NRORG AND IDAPLICATIVO = 'MYMENU'";
    	        		
    	        $queryParams = [
	                "P_DSDNS" 		=> $row["DSDNS"],
	                "P_DSPORTA"		=> $row["DSPORTA"],
	                "P_DTULTATU"	=> date("d-m-Y H:m:s"),
	                "P_NRORG" 		=> $row["NRORG"]
	            ];
    	    }
    	    else {
        	    $row["NRAPPCONFIG"] = $this->getNextId("NRAPPCONFIG", "APPCONFIG");
        	    
        	    $query = "INSERT INTO APPCONFIG(NRAPPCONFIG, DSDNS, DSPORTA, IDAPLICATIVO, IDATIVO, DTINCLUSAO, DTULTATU, NRORG, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU) "
        	    		."VALUES (:P_NRAPPCONFIG, :P_DSDNS, :P_DSPORTA, :P_IDAPLICATIVO, :P_IDATIVO, :P_DTINCLUSAO, :P_DTULTATU, :P_NRORG, :P_NRORGINCLUSAO, :P_CDOPERINCLUSAO, :P_NRORGULTATU, :P_CDOPERULTATU)";
        	    
				$queryParams = [
					"P_NRAPPCONFIG" 	=> $row["NRAPPCONFIG"],
					"P_DSDNS"			=> $row["DSDNS"],
					"P_DSPORTA"			=> $row["DSPORTA"],
					"P_IDAPLICATIVO"	=> "MYMENU",
					"P_IDATIVO"			=> "S",
					"P_DTINCLUSAO"		=> date("d-m-Y H:m:s"),
					"P_DTULTATU"		=> date("d-m-Y H:m:s"),
					"P_NRORG"			=> $row["NRORG"],
					"P_NRORGINCLUSAO" 	=> 1.0,
					"P_CDOPERINCLUSAO"	=> "1",
					"P_NRORGULTATU"		=> 1,
					"P_CDOPERULTATU"	=> "000000000999"
				];
    	    }
    	    
    	    $statement = $this->connection->prepare($query);
    	    $statement->execute($queryParams);
    	    
    	    $response->addDataSet(new DataSet("data",["Salvo com sucesso"]));
	    } catch(\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
	}

	public function saveForm(Request $request, Response $response) {
		try {
			$row = $request->getRow();
			
			if ($row["NRQRCODELOCALPESQ"]) {
				$query = "UPDATE QRCODELOCALPESQ "
						."SET NRLOCALPESQ = :P_NRLOCALPESQ, "
						."	  DSQRCODEAPP = :P_DSQRCODEAPP, " 
						."	  QTDIASVALIDADE = :P_QTDIASVALIDADE, "
						."	  DTEXPIRACAO = :P_DTEXPIRACAO, "
						."	  DTULTATU = :P_DTULTATU "
						."WHERE NRQRCODELOCALPESQ = :P_NRQRCODELOCALPESQ"
						."	AND NRORG = :P_NRORG";
				
				$queryParams = [
					"P_NRQRCODELOCALPESQ"	=> $row["NRQRCODELOCALPESQ"],
					"P_NRORG"				=> $row["NRORG"],
					"P_NRLOCALPESQ" 		=> $row["NRLOCALPESQ"],
					"P_DSQRCODEAPP" 		=> $row["DSQRCODEAPP"],
					"P_QTDIASVALIDADE"		=> $row["QTDIASVALIDADE"],
					"P_DTEXPIRACAO" 		=> $row["DTEXPIRACAO"],	
					"P_DTULTATU"			=> date("d-m-Y H:m:s")
				];
			}
			else {
				$row["NRQRCODELOCALPESQ"] = $this->getNextId("NRQRCODELOCALPESQ", "QRCODELOCALPESQ");
				
				$query = "INSERT INTO QRCODELOCALPESQ(NRQRCODELOCALPESQ, CDFILIAL, NRLOCALPESQ, DSQRCODEAPP, QTDIASVALIDADE, DTEXPIRACAO, IDATIVO, DTINCLUSAO, DTULTATU, NRORG, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU) "
						."VALUES (:P_NRQRCODELOCALPESQ, :P_CDFILIAL, :P_NRLOCALPESQ, :P_DSQRCODEAPP, :P_QTDIASVALIDADE, :P_DTEXPIRACAO, :P_IDATIVO, :P_DTINCLUSAO, :P_DTULTATU, :P_NRORG, :P_NRORGINCLUSAO, :P_CDOPERINCLUSAO, :P_NRORGULTATU, :P_CDOPERULTATU)";
				
				$queryParams = [
					"P_NRQRCODELOCALPESQ"	=> $row["NRQRCODELOCALPESQ"],
					"P_CDFILIAL"			=> $row["CDFILIAL"],
					"P_NRLOCALPESQ" 		=> $row["NRLOCALPESQ"],
					"P_DSQRCODEAPP" 		=> $row["DSQRCODEAPP"],
					"P_QTDIASVALIDADE"		=> $row["QTDIASVALIDADE"],
					"P_DTEXPIRACAO" 		=> $row["DTEXPIRACAO"],
					"P_IDATIVO" 			=> "S",
					"P_DTINCLUSAO"			=> date("d-m-Y H:m:s"),
					"P_DTULTATU"			=> date("d-m-Y H:m:s"),
					"P_NRORG"				=> $row["NRORG"],
					"P_NRORGINCLUSAO"		=> 1.0,
					"P_CDOPERINCLUSAO"		=> "1",
					"P_NRORGULTATU" 		=> 1,
					"P_CDOPERULTATU"		=> 1
				];
			}
				
			$statement = $this->connection->prepare($query);
			$statement->execute($queryParams);
			
		    $response->addDataSet(new DataSet("data",["Salvo com sucesso", $row["NRQRCODELOCALPESQ"]]));
		} catch(\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
	}
	
	public function delete(Request $request, Response $response) {
		try {
			$row = $request->getRow();
			
			$query = "DELETE FROM QRCODELOCALPESQ "
					."WHERE NRQRCODELOCALPESQ = :P_NRQRCODELOCALPESQ "
					."	AND NRORG = :P_NRORG";
			$queryParams = [
				"P_NRQRCODELOCALPESQ" => $row["NRQRCODELOCALPESQ"],
				"P_NRORG"			  => $row["NRORG"]
			];
			
			$statement = $this->connection->prepare($query);
			$statement->execute($queryParams);
			
			$response->addDataSet(new DataSet("data", ["SUCESSO_EXCLUSAO"]));
		} catch(\Exception $e) {
			$message = $e->getMessage();
			$errorMessage = "";
			
			if(stripos($message, "ORA-02292")) {
				$errorMessage = "DELETING_ERROR_INTEGRITY_CONSTRAINT";
			} else {
				$errorMessage = "UNEXPECTED_ERROR_DELETING";
			}
			
			$response->setError(new Error($errorMessage, 500));
			$response->addDataSet(new DataSet("deletingError", array('ERROR' => $e->getMessage())));
		}

	}

	public function save(DTO\Request $request, DTO\Response $response){
		try {
			$this->dataSourceName = 'qrcodelocalpesq';
			$rows = $this->crudService->getRowsToSave($request, $this->dataSourceName);
			$rowsToSave = [];
			foreach($rows as $row) {
			    $imageCol = array_key_exists("URLLOGO", $row) ? $row['URLLOGO'] : false;
			    if ($imageCol && gettype($imageCol) !== "string") {
			        $imageColValue = null;
    			    if (gettype($imageCol) === "array") {
    			        if (sizeof($imageCol) > 0) {
    			            $fileObject = $imageCol[0];
    			            if (gettype($fileObject) === "array" && array_key_exists("type", $fileObject) && array_key_exists("path", $fileObject)) {
        			            $fileType = $fileObject['type'];
        			            if (substr($fileType, 0, 5) === "image") {
        			                $imageColValue = $fileObject['path'];
        			            }
    			            }
    			        }
					}
    			    $row['URLLOGO'] = $imageColValue;
			    } else if ($imageCol && gettype($imageCol) === "string") {
			       $row['URLLOGO'] = str_replace($this->fileServerFilesUrl, "", $row['URLLOGO']);
			    }
			    array_push($rowsToSave, $row);
			}
			$this->crudService->save($this->dataSourceName, $rowsToSave);
            $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
    }

	public function find(DTO\Request\Filter $request, DTO\Response $response) {
        try {
			$this->dataSourceName = "parametrizacao_qrcode";
            $filterCriteria = $request->getFilterCriteria();
            $filterCriteria->setDataSourceName($this->dataSourceName);
            $filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
            $newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, $this->parameterBagColumns);
            $this->crudService->find($response, $newFilterCriteria);
	        
	        $responseDataSourceName = substr($request->getRoutePath(), 1);
	        $responseRows = [];
	        foreach ($response->getDataSets()[0]->getRows() as $row) {
	            $imageCol = array_key_exists("URLLOGO", $row) ? $row['URLLOGO'] : false;
			    if ($imageCol && gettype($imageCol) === "string") {
    			    $row['URLLOGO'] = $this->fileServerFilesUrl.$row['URLLOGO'];
				}
	            array_push($responseRows, $row);
	        }
	        
	        $responseDataSet = new DataSet($responseDataSourceName, $responseRows);
	        $response->setDataSets([$responseDataSet]);
	        
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}